const fileElement: HTMLInputElement = document.querySelector('#myFile');
const formElement: HTMLFormElement = document.querySelector('#first');
const titleElement: HTMLInputElement = document.querySelector('#title');
const canvasElement: HTMLCanvasElement = document.querySelector('#second');
const imageElement: HTMLImageElement = document.querySelector('#myImage');


formElement.addEventListener('submit', (event) => {
    event.preventDefault()
    const file = fileElement.files[0];
    const reader = new FileReader();
    reader.onload = () => showImage(reader);
    reader.readAsDataURL(file);
    document.title = titleElement.value;

    formElement.style.display = 'none';
    canvasElement.style.display = 'block';
})


function showImage(fileReader) {
    imageElement.onload = () => {
        const y = getImageData(imageElement);
        // console.log(y);
        canvasElement.width = imageElement.width;
        canvasElement.height = imageElement.height - y;
        const ctx = canvasElement.getContext('2d');
        ctx.drawImage(imageElement, 0, y, canvasElement.width, canvasElement.height, 0, 0, canvasElement.width, canvasElement.height);
        //
        // ctx.drawImage(img, 0, y);
        // imageElement.style.height = `calc(100vh + ${y}px)`;
        // imageElement.style['object-position'] = `0 -${y}px`;
    };
    imageElement.src = fileReader.result;
}

function getPixelOfCoordinate(imageData: Uint8ClampedArray, imageWidth: number, x: number, y: number) {
    const index = (x + y * imageWidth) * 4;
    return imageData.slice(index, index + 4);
}

function getWholeLine(imageData: Uint8ClampedArray, imageWidth: number, y: number) {
    const resultLine: Uint8ClampedArray[] = [];
    for (let x = 0; x < imageWidth; x++) {
        resultLine.push(getPixelOfCoordinate(imageData, imageWidth, x, y));
    }
    return resultLine;
}

function getAllLinesOfImage(imageData: Uint8ClampedArray, imageWidth: number, imageHeight: number) {
    const resultLines: Uint8ClampedArray[][] = [];
    for (let y = 0; y < imageHeight; y++) {
        resultLines.push(getWholeLine(imageData, imageWidth, y));
    }
    return resultLines;
}

function pixelEqual(a: Uint8ClampedArray, b: Uint8ClampedArray) {
    return a.every((value, index) => b[index] == value);
}

function lineEqual(a: Uint8ClampedArray[], b: Uint8ClampedArray[]) {
    return a.every((value, index) => pixelEqual(value, b[index]));
}

function lineEqualAny(a: Uint8ClampedArray[], b: Uint8ClampedArray[]) {
    return a.some((value, index) => pixelEqual(value, b[index]));
}

function isWholeLineSameColor(line: Uint8ClampedArray[]) {
    const firstPixel = line[0];
    return line.every(pixel => pixelEqual(pixel, firstPixel));
}



function getImageData(img) {
    const canvas = document.createElement("canvas");
    canvas.width = img.width;
    canvas.height = img.height;
    const ctx = canvas.getContext("2d");
    ctx.drawImage(img, 0, 0);
    const imageData = ctx.getImageData(0, 0, img.width, img.height).data;

    const allLines = getAllLinesOfImage(imageData, img.width, img.height);

    const firstPositionOfBackground = allLines.findIndex(isWholeLineSameColor);
    const backGroundPixel = allLines[firstPositionOfBackground][0];
    console.log(backGroundPixel);
    const firstPositionOfPicture = allLines.findIndex(line => line.every(pixel => !pixelEqual(pixel, backGroundPixel)));
    console.log(firstPositionOfPicture);
    return firstPositionOfPicture;
    /*
    // console.log(imageData);
    // console.log(imageData.length, imageData.byteLength)
    // RGBA format
    const bgColor = imageData.subarray(0, 4);
    // let index = 0;
    for (let y = 0; y < img.height; y++) {
        let flag = false;
        for (let x = 0; x < img.width; x++) {
            const index = (x + y * img.width) * 4;
            const currentPixel = imageData.slice(index, index + 4);
            // console.log(bgColor, currentPixel, index)
            if (pixelEqual(bgColor, currentPixel)) {
                flag = true;
                // console.log(`line ${y} get a same`)
                break;
            }
        }
        if (!flag) {
            return y;
        }
    }
    // console.log(bgColor);
    // console.log("image data:", imageData);
    */
}
